# -*- coding: utf-8 -*-

# Copyright(C) 2012-2014 Florent Fourcot
# Copyright(C) 2018      Eric Seigne
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.capabilities.bill import DocumentTypes, CapDocument, SubscriptionNotFound, DocumentNotFound, Subscription, Bill
from weboob.capabilities.base import find_object
from weboob.tools.backend import Module, BackendConfig
from weboob.tools.value import ValueBackendPassword, Value

from .browser import UlysBrowser

__all__ = ['UlysModule']


class UlysModule(Module, CapDocument):
    NAME = 'ulys'
    DESCRIPTION = u'Ulys Vinci Autoroute'
    MAINTAINER = u'Eric Seigne'
    EMAIL = 'weboob@cap-rel.fr'
    LICENSE = 'AGPLv3+'
    VERSION = '1.5'
    CONFIG = BackendConfig(Value('login', label='Account ID'),
                           ValueBackendPassword('password', label='Password'));

    BROWSER = UlysBrowser

    accepted_document_types = (DocumentTypes.BILL,)

    def create_default_browser(self):
        self.logger.debug('call Module.create_default_browser')
        return self.create_browser(self.config)

    def iter_subscription(self):
        self.logger.debug('call Module.iter_subscription')
        return self.browser.get_subscription_list()

    def get_subscription(self, _id):
        self.logger.debug('call Module.get_subscription')
        if not _id.isdigit():
            raise SubscriptionNotFound()
        subscription = self.browser.get_subscription(_id)
        if not subscription:
            raise SubscriptionNotFound()
        else:
            return subscription

    def iter_documents_history(self, subscription):
        self.logger.debug('call Module.iter_documents_history')
        if not isinstance(subscription, Subscription):
            subscription = self.get_subscription(subscription)
        return self.browser.iter_history(subscription)

    def get_details(self, subscription):
        self.logger.debug('call Module.get_details')
        if not isinstance(subscription, Subscription):
            subscription = self.get_subscription(subscription)
        return self.browser.get_details(subscription)

    def iter_documents(self, subscription):
        self.logger.debug('call Module.iter_document')
        if not isinstance(subscription, Subscription):
            subscription = self.get_subscription(subscription)
        return self.browser.iter_documents(subscription)

    def get_document(self, id):
        self.logger.debug('call Module.get_document')
        bill = self.browser.get_document(id)
        if not bill:
            raise DocumentNotFound()
        else:
            return bill

    def download_document(self, bill):
        self.logger.debug('call Module.download_document')
        if not isinstance(bill, Bill):
            bill = self.get_document(bill)
        response = self.browser.open(bill.url, stream=True)
        if not response :
            return None
        return response.content
